// Copyright (c) 2012 ARM Ltd
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// ---------------------------------------------------------------------------
// This script runs in the main (application) process and has the following
// responsibilities:
//
//    * Load the frame script (content.js) into each content page.
//
//    * Listen for annotation messages from content processes and send them to
//      Gator by writing to /dev/gator/annotate.
// ---------------------------------------------------------------------------

var ARMGator = {
    onLoad : function(aEvent) {
        // Register an message listener so the content processes can send
        // annotations to this object.
        window.messageManager.addMessageListener("ARMGator:Annotate", this);

        // Load the frame script into the child process context, and configure
        // it to load in all future loaded pages (such as new tabs).
        //
        // The content.js script allows content JavaScript code to send
        // annotations to ARMGator.annotate using the message-passing API.
        window.messageManager.loadFrameScript("chrome://armgator/content/content.js", true);
    },

    // Handle messages from the content scripts.
    receiveMessage : function(aMessage) {
        // Pass on aMessage to the annotate function.
        this.annotate(aMessage.json.message, aMessage.json.colour);
    },

    // This is a simple utility function that forces the input to an integer in
    // the range 0-255 (inclusive).
    get_colour_byte : function(channel) {
        let code = Math.floor(channel);
        if (code > 255) {
            code = 255;
        }
        if (code < 0) {
            code = 0;
        }
        return code;
    },

    // Write an annotation to the annotation file.
    //
    // This function will work if called directly, but is usually called via
    // the message-passing API.
    //
    //  * The message argument should be a simple string, to be used as the
    //    annotation message. If this is undefined or is an empty string, any
    //    existing annotation will be terminated.
    //
    //  * The colour argument should be an object with 'red', 'green' and
    //    'blue' properties, each with values 0-255 to represent channel
    //    intensity. If this is undefined, no colour code will be sent and
    //    Streamline will be used instead.
    annotate : function(message, colour) {
        if (message === undefined) {
            // Set a default message.
            message = "";
        }

        // Load the annotation file-name from a preference. By default, this
        // will be "/dev/gator/annotate" on Linux and Android.
        let filename = Services.prefs.getCharPref("extensions.armgator.annotation_file");

        // If the Gator integration is enabled, write the Gator annotation.
        if (Services.prefs.getBoolPref("extensions.armgator.enable_gator")) {
            // Get a reference to the annotation file.
            let file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
            file.initWithPath(filename);

            // Open the annotation file as an nsIFileOutputStream with the
            // following flags:
            //
            //    * Write access (0x02).
            //    * Synchronous mode (0x40).
            //    * Truncate existing file (0x20).
            //
            // Synchronous mode is used to minimize the delay between sending
            // an annotation and it being seen by Gator (and thus Streamline).
            //
            // If the file doesn't already exist Gator isn't listening anyway,
            // so to be cautious don't automatically create it if it's missing.
            let foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(file, 0x02|0x20|0x40, 0666, 0);

            // Write the colour code, if one was specified, at the start of the
            // annotation file.
            if (colour !== undefined) {
                let code = String.fromCharCode(
                        0x1b,   // ASCII escape character.
                        this.get_colour_byte(colour.red),
                        this.get_colour_byte(colour.green),
                        this.get_colour_byte(colour.blue));
                // Write the colour code as binary data.
                foStream.write(code, 4);
            }
            
            // After writing the binary data (if a colour was specified),
            // create a UTF-8 converter to write the message to the output
            // stream. A NULL character is also added to terminate the message,
            // as required by Gator.
            let converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
            converter.init(foStream, "UTF-8", 0, 0);
            converter.writeString(message + "\x00");
            converter.close();  // This automatically closes foStream.
        }

        // If the ARMGator alert mechanism is enabled, sent an alert
        // notification.
        if (Services.prefs.getBoolPref("extensions.armgator.enable_alert")) {
            let alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
            alerts.showAlertNotification(null, filename, message, false, "", null);
        }
    }
};

// Register the ARMGator.onLoad event listener.
window.addEventListener("load", function(e) { ARMGator.onLoad(e); }, false);

